package com.bizmatics.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.bizmatics.common.core.bean.CommonPage;
import com.bizmatics.common.mvc.base.AbstractCrudService;
import com.bizmatics.model.DeviceAnalogVariableList;
import com.bizmatics.model.DeviceAttribute;
import com.bizmatics.model.system.SysUser;
import com.bizmatics.model.vo.MonitorDeviceListVO;
import com.bizmatics.persistence.mapper.DeviceAttributeMapper;
import com.bizmatics.service.DeviceAttributeService;
import com.bizmatics.service.DeviceService;
import com.bizmatics.service.util.SecurityUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;

/**
 * 设备属性 服务实现类
 *
 * @author ya
 * @since 2021-09-24
 */
@SuppressWarnings("unused")
@Service
public class DeviceAttributeServiceImpl extends AbstractCrudService<DeviceAttributeMapper, DeviceAttribute> implements DeviceAttributeService {

    @Autowired
    private DeviceService deviceService;

    @Override
    public void deviceNewsAdd(DeviceAttribute deviceAttribute) {
        SysUser user = SecurityUtils.getLoginUser().getUser();
        deviceAttribute.setStatus(1);
        deviceAttribute.setCreator(user.getUserName());
        deviceAttribute.setCreateTime(new Date());
        this.save(deviceAttribute);
    }

    @Override
    public void deviceNewsUpdate(DeviceAttribute deviceAttribute) {
        this.updateById(deviceAttribute);
    }

    @Override
    public  CommonPage<DeviceAttribute> deviceNewsList(Integer id, Integer siteId,Integer size,Integer current) {
        IPage<DeviceAttribute> page = new Page<DeviceAttribute>(current, size);
        LambdaQueryWrapper<DeviceAttribute> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(DeviceAttribute::getStatus, 1).eq(DeviceAttribute::getSiteId, siteId);
        if (id != 0) {
            queryWrapper.eq(DeviceAttribute::getId, id);
        }
        page = this.page(page, queryWrapper);
        this.ToCommonPage(page);
        return new CommonPage<>(page.getRecords(), page.getTotal(), page.getSize(),page.getCurrent());
    }

    @Override
    public void deviceNewsDel(int id) {
        DeviceAttribute deviceAttribute = new DeviceAttribute();
        deviceAttribute.setId(id);
        deviceAttribute.setStatus(0);
        this.updateById(deviceAttribute);
    }

    @Override
    public CommonPage<MonitorDeviceListVO> monitorDeviceList(String siteName ,Integer size,Integer current) {
        List<MonitorDeviceListVO> monitorDeviceListOne = baseMapper.monitorDeviceList(siteName,null,0);
        Integer total = 0;
        if (monitorDeviceListOne.size()>0){
            total = monitorDeviceListOne.size();
        }
        Integer startCurrent = (current - 1) * size;
        List<MonitorDeviceListVO> monitorDeviceList = baseMapper.monitorDeviceList(siteName,startCurrent,size);
        return new CommonPage<>(monitorDeviceList, total, size,current);
    }
}
