package com.bizmatics.service.impl;

import com.bizmatics.common.core.bean.CommonPage;
import com.bizmatics.common.mvc.base.AbstractCrudService;
import com.bizmatics.model.WcBlackoutPlan;
import com.bizmatics.model.system.SysUser;
import com.bizmatics.model.vo.CorrespondDeviceVO;
import com.bizmatics.model.vo.PlannedOutageVo;
import com.bizmatics.model.vo.WcBlackoutPlanVo;
import com.bizmatics.persistence.mapper.WcBlackoutPlanMapper;
import com.bizmatics.service.WcBlackoutPlanService;
import com.bizmatics.service.util.SecurityUtils;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * 计划停电
 *
 * @author ya
 * @since 2021-07-07
 */
@Service
public class WcBlackoutPlanServiceImpl extends AbstractCrudService<WcBlackoutPlanMapper, WcBlackoutPlan> implements WcBlackoutPlanService {

    @Override
    public void plannedOutageAdd(WcBlackoutPlan wcBlackoutPlan) {
        SysUser user = SecurityUtils.getLoginUser().getUser();
        wcBlackoutPlan.setEnable(1);
        wcBlackoutPlan.setCreator(user.getUserName());
        wcBlackoutPlan.setCreateTime(new Date());
        this.save(wcBlackoutPlan);
    }

    @Override
    public void plannedOutageUpdate(WcBlackoutPlan wcBlackoutPlan) {
        this.updateById(wcBlackoutPlan);
    }

    @Override
    public void plannedOutageDel(int id) {
        WcBlackoutPlan wcBlackoutPlan = new WcBlackoutPlan();
        wcBlackoutPlan.setId(id);
        wcBlackoutPlan.setEnable(0);
        this.updateById(wcBlackoutPlan);
    }

    @Override
    public CommonPage<WcBlackoutPlanVo> plannedOutageList(PlannedOutageVo plannedOutageVo) {
        List<WcBlackoutPlanVo> wcBlackoutPlanVoListOne = baseMapper.plannedOutageList(plannedOutageVo.getStartTime(), plannedOutageVo.getEndTime(), plannedOutageVo.getType(), plannedOutageVo.getId(),plannedOutageVo.getSize(),null);
        int total = 0;
        if (wcBlackoutPlanVoListOne.size() > 0) {
            total = wcBlackoutPlanVoListOne.size();
        }
        int startCurrent = (plannedOutageVo.getCurrent() - 1) * plannedOutageVo.getSize();

        List<WcBlackoutPlanVo> plannedOutageList = baseMapper.plannedOutageList(plannedOutageVo.getStartTime(), plannedOutageVo.getEndTime(), plannedOutageVo.getType(), plannedOutageVo.getId(),plannedOutageVo.getSize(),startCurrent);
        List<WcBlackoutPlanVo> plannedOutageListOne = new ArrayList<>();
        if (plannedOutageList.size()>0){
            for (int i = 0; i < plannedOutageList.size(); i++) {
                WcBlackoutPlanVo wcBlackoutPlanVo = new WcBlackoutPlanVo();
                wcBlackoutPlanVo.setSiteId(plannedOutageList.get(i).getSiteId());
                wcBlackoutPlanVo.setPlanType(plannedOutageList.get(i).getPlanType());
                wcBlackoutPlanVo.setStartTime(plannedOutageList.get(i).getStartTime());
                wcBlackoutPlanVo.setEndTime(plannedOutageList.get(i).getEndTime());
                wcBlackoutPlanVo.setContacts(plannedOutageList.get(i).getContacts());
                wcBlackoutPlanVo.setPhone(plannedOutageList.get(i).getPhone());
                wcBlackoutPlanVo.setEnable(plannedOutageList.get(i).getEnable());
                wcBlackoutPlanVo.setCreator(plannedOutageList.get(i).getCreator());
                wcBlackoutPlanVo.setCreateTime(plannedOutageList.get(i).getCreateTime());
                wcBlackoutPlanVo.setSiteName(plannedOutageList.get(i).getSiteName());
                wcBlackoutPlanVo.setId(plannedOutageList.get(i).getId());
                if (plannedOutageList.get(i).getStartTime().getTime()>new Date().getTime()){
                    wcBlackoutPlanVo.setType(1);
                }else if(plannedOutageList.get(i).getStartTime().getTime()<new Date().getTime()&&plannedOutageList.get(i).getEndTime().getTime()>new Date().getTime()){
                    wcBlackoutPlanVo.setType(2);
                }else if (plannedOutageList.get(i).getEndTime().getTime()<new Date().getTime()){
                    wcBlackoutPlanVo.setType(3);
                }
                plannedOutageListOne.add(wcBlackoutPlanVo);
            }
        }
        return new CommonPage<>(plannedOutageListOne, total, plannedOutageVo.getSize(), plannedOutageVo.getCurrent());
    }
}
